#!/usr/bin/python
'''Numerical calculation of hyperbolic length along a grafting ray, by David Dumas'''

VERSION='0.0.2'

import sys

opts = { 'words': ['a'], 'gap': None, 'res': None, 'bound': None }

def showextra():
    sys.stderr.write('Extra options and default values:\n')
    for o in opts:
        sys.stderr.write('\t%s=%s\n' % (str(o),str(opts[o])))

if len(sys.argv) < 4:
    sys.stderr.write('Grafting length calculator version %s\n' % VERSION)
    sys.stderr.write('Usage: %s TX TY THETA [extra opts, KEY=VALUE]\n' % sys.argv[0])
    showextra()
    sys.exit(1)

tx = float(sys.argv[1])
ty = float(sys.argv[2])
theta = float(sys.argv[3])
opts = { 'words': ['a'], 'gap': None, 'res': None, 'bound': None }

if len(sys.argv) > 4:
    for a in sys.argv[4:]:
        k,v = a.split('=')
        if k in opts:
            if k == 'words':
                opts[k] = v.strip('\'\"').split()
            else:
                opts[k] = v
            sys.stderr.write('Set option: %s=%s\n' % (k,opts[k]))
        else:
            sys.stderr.write('Unrecognized option %s.\n' % k)
            showextra()
            sys.exit(1)

from kleinian import *
from zipper import *
from freegroup import *
from cmath import *
from numpy import *
from numpy import linalg

def ztow(ztrip,wtrip):
    z1,z2,z3 = ztrip
    w1,w2,w3 = wtrip
    a=w1*(w2*(z2-z1) + w3*(z1-z3)) + w2*w3*(z3-z2)
    b=w2*w3*z1*(z2 - z3) + w1*(w2*(z1-z2)*z3 + w3*z2*(z3-z1))
    c=-(w3*(z2-z1) + w2*(z1-z3) + w1*(z3-z2))
    d=w2*z2*(z1-z3) + w3*(z2-z1)*z3 + w1*z1*(z3-z2)
    return matrix([[a,b],[c,d]])

def mobius(m,z):
    v = m*matrix([[complex(z)],[complex(1,0)]])
    return v[0,0]/v[1,0]

def getz(x,y):
    return 0.5 * (x * y - sqrt(x*x*y*y - 4.0*(x*x + y*y)))

def getfixa(x,y,z):
    return (sqrt((4.0j - 2.0*y + x*z)*(-2.0*x + (2.0j + y)*z)**2) /
          sqrt(-((y**2)*(4.0j + 2*y - x*z)*(-4.0 + z**2))))

def bendtrb(x,y,z,theta):
    F = getfixa(x,y,z)
    return y*cos(0.5*theta) - (1.0j*(4.0*x*x - 4.0*x*y*z + (4.0 + y*y)*z*z + F*F*y*y*(-4.0 + z*z))*sin(0.5*theta))/(2.0*F*y*(-4.0 + z*z))

K = Kleinian(tx,bendtrb(tx,ty,getz(tx,ty),theta),getz(tx,ty),gap=opts['gap'],res=opts['res'],bound=opts['bound'])
minabs = 10.0
for z in K.limset:
    a = abs(complex(z[0],z[1])) 
    if a < minabs:
        minabs = a

F = FreeGroup({'a': K.A, 'b':K.B}) 
triple = [complex(-0.5*minabs,0), complex(0,0), complex(0.5*minabs,0) ]
points = triple
for w in opts['words']:
    itriple = [ mobius(F[w],z) for z in triple ]
    points = points + itriple 

points = [ (z.real,z.imag) for z in points ]
Z = Zipper(K.limset,[0,0],points)
ntriple = [ complex(v[0],v[1]) for v in Z.images[:3] ]
restimg = [ complex(v[0],v[1]) for v in Z.images[3:] ]
for w in opts['words']:
    nimg = restimg[:3]
    restimg = restimg[3:]
    # The Kleinian group
    t = trace(F[w])
    d = linalg.det(F[w])
    L = 2.0 * arccosh(t/(2.0*sqrt(d)))
    sys.stdout.write(' %f' % L.real)
    # The Fuchsian group
    Afuchs = ztow(ntriple,nimg)
    t = trace(Afuchs)
    d = linalg.det(Afuchs)
    L = 2.0 * arccosh(t/(2.0*sqrt(d)))
    sys.stdout.write(' %f' % L.real)
sys.stdout.write('\n')
